"use strict";
exports.__esModule = true;
exports.all = void 0;
var tslib_1 = require("tslib");
var experimental_utils_1 = require("@typescript-eslint/experimental-utils");
var typescript_1 = tslib_1.__importDefault(require("typescript"));
var compare_versions_1 = require("compare-versions");
var createRule = experimental_utils_1.ESLintUtils.RuleCreator(function (ruleName) { return ruleName; });
// ts 3.9 introduce ts-expect-error comment.
// @see https://devblogs.microsoft.com/typescript/announcing-typescript-3-9-beta/
var defaultType = compare_versions_1.compare(typescript_1["default"].versionMajorMinor, "3.9", ">=") ? "ts-expect-error" : "ts-ignore";
var defaultOptions = {
    tsCommentType: defaultType,
    reportOnly: false
};
exports.all = createRule({
    name: "all",
    meta: {
        type: "suggestion",
        docs: {
            description: "show tsc error plugin.",
            category: "Best Practices",
            recommended: false,
            requiresTypeChecking: true
        },
        messages: {
            // see: https://eslint.org/docs/developer-guide/working-with-rules#using-message-placeholders
            message: "{{ message }}"
        },
        schema: [
            {
                type: "object",
                properties: {
                    tsCommentType: {
                        type: "string",
                        "enum": ["ts-ignore", "ts-expect-error"]
                    },
                    reportOnly: {
                        type: "boolean"
                    }
                },
                additionalProperties: false
            },
        ],
        fixable: "code"
    },
    defaultOptions: [defaultOptions],
    create: function (context) {
        var program = experimental_utils_1.ESLintUtils.getParserServices(context).program;
        // eslint-disable-next-line @typescript-eslint/no-empty-function
        var emitResults = program.emit(undefined, function () { });
        var allDiagnostics = tslib_1.__spreadArrays(typescript_1["default"].getPreEmitDiagnostics(program, program.getSourceFile(context.getFilename())), emitResults.diagnostics);
        return {
            "Program:exit": function () {
                allDiagnostics.forEach(function (diagnostic) {
                    if (diagnostic.file) {
                        var _a = diagnostic.file.getLineAndCharacterOfPosition(diagnostic.start), startLine = _a.line, startCharacter = _a.character;
                        var _b = diagnostic.file.getLineAndCharacterOfPosition(diagnostic.start + diagnostic.length), endLine = _b.line, endCharacter = _b.character;
                        var message = typescript_1["default"].flattenDiagnosticMessageText(diagnostic.messageText, "\n");
                        var firstOfLineToken_1 = diagnostic.file.getPositionOfLineAndCharacter(startLine, 0);
                        var messageForComment_1 = typescript_1["default"].flattenDiagnosticMessageText(diagnostic.messageText, "; ");
                        var reportOnly_1 = context.options[0].reportOnly;
                        // TODO: filter error via typescript error codes.
                        context.report({
                            loc: {
                                start: {
                                    line: startLine,
                                    column: startCharacter
                                },
                                end: {
                                    line: endLine,
                                    column: endCharacter
                                }
                            },
                            messageId: "message",
                            data: {
                                message: message
                            },
                            fix: (function () {
                                if (reportOnly_1) {
                                    return null;
                                }
                                return function (fixer) {
                                    var tsCommentType = context.options[0].tsCommentType;
                                    var comment = "// @" + tsCommentType + " with: " + messageForComment_1 + "\n";
                                    return fixer.insertTextBeforeRange([firstOfLineToken_1, firstOfLineToken_1], comment);
                                };
                            })()
                        });
                    }
                });
            }
        };
    }
});
//# sourceMappingURL=all.js.map